@@ -1033,6 +1033,9 @@ func (uart *UART) Configure(config UARTConfig) error {
 	uart.Bus.CTRLA.Set((1 << sam.SERCOM_USART_INT_CTRLA_MODE_Pos) |
 		(1 << sam.SERCOM_USART_INT_CTRLA_SAMPR_Pos)) // sample rate of 16x
 
+	// set clock
+	setSERCOMClockGenerator(uart.SERCOM, sam.GCLK_PCHCTRL_GEN_GCLK1)
+
 	// Set baud rate
 	uart.SetBaudRate(config.BaudRate)
 
@@ -1124,7 +1127,8 @@ type I2CConfig struct {
 
 const (
 	// SERCOM_FREQ_REF is always reference frequency on SAMD51 regardless of CPU speed.
-	SERCOM_FREQ_REF = 48000000
+	SERCOM_FREQ_REF       = 48000000
+	SERCOM_FREQ_REF_GCLK0 = 120000000
 
 	// Default rise time in nanoseconds, based on 4.7K ohm pull up resistors
 	riseTimeNanoseconds = 125
@@ -1178,6 +1182,9 @@ func (i2c *I2C) Configure(config I2CConfig) error {
 		i2c.Bus.SYNCBUSY.HasBits(sam.SERCOM_I2CM_SYNCBUSY_SWRST) {
 	}
 
+	// set clock
+	setSERCOMClockGenerator(i2c.SERCOM, sam.GCLK_PCHCTRL_GEN_GCLK1)
+
 	// Set i2c controller mode
 	//SERCOM_I2CM_CTRLA_MODE( I2C_MASTER_OPERATION )
 	// sam.SERCOM_I2CM_CTRLA_MODE_I2C_MASTER = 5?
@@ -1483,8 +1490,18 @@ func (spi SPI) Configure(config SPIConfig) error {
 		spi.Bus.CTRLA.ClearBits(sam.SERCOM_SPIM_CTRLA_CPOL)
 	}
 
+	// set clock
+	freqRef := uint32(0)
+	if config.Frequency > SERCOM_FREQ_REF/2 {
+		setSERCOMClockGenerator(spi.SERCOM, sam.GCLK_PCHCTRL_GEN_GCLK0)
+		freqRef = uint32(SERCOM_FREQ_REF_GCLK0)
+	} else {
+		setSERCOMClockGenerator(spi.SERCOM, sam.GCLK_PCHCTRL_GEN_GCLK1)
+		freqRef = uint32(SERCOM_FREQ_REF)
+	}
+
 	// Set synch speed for SPI
-	baudRate := SERCOM_FREQ_REF / (2 * config.Frequency)
+	baudRate := freqRef / (2 * config.Frequency)
 	if baudRate > 0 {
 		baudRate--
 	}
@@ -1965,11 +1982,13 @@ type USBCDC struct {
 	waitTxc           bool
 	waitTxcRetryCount uint8
 	sent              bool
+	configured        bool
 }
 
 var (
 	// USB is a USB CDC interface.
-	USB = &USBCDC{Buffer: NewRingBuffer()}
+	USB        = &USBCDC{Buffer: NewRingBuffer()}
+	waitHidTxc bool
 )
 
 const (
@@ -2149,6 +2168,13 @@ func (usbcdc *USBCDC) Configure(config UARTConfig) {
 	interrupt.New(sam.IRQ_USB_SOF_HSOF, handleUSBIRQ).Enable()
 	interrupt.New(sam.IRQ_USB_TRCPT0, handleUSBIRQ).Enable()
 	interrupt.New(sam.IRQ_USB_TRCPT1, handleUSBIRQ).Enable()
+
+	usbcdc.configured = true
+}
+
+// Configured returns whether usbcdc is configured or not.
+func (usbcdc *USBCDC) Configured() bool {
+	return usbcdc.configured
 }
 
 func handlePadCalibration() {
@@ -2216,6 +2242,9 @@ func handleUSBIRQ(interrupt.Interrupt) {
 	// Start of frame
 	if (flags & sam.USB_DEVICE_INTFLAG_SOF) > 0 {
 		USB.Flush()
+		if hidCallback != nil && !waitHidTxc {
+			hidCallback()
+		}
 		// if you want to blink LED showing traffic, this would be the place...
 	}
 
@@ -2239,6 +2268,9 @@ func handleUSBIRQ(interrupt.Interrupt) {
 			// Class Interface Requests
 			if setup.wIndex == usb_CDC_ACM_INTERFACE {
 				ok = cdcSetup(setup)
+			} else if setup.bmRequestType == usb_SET_REPORT_TYPE && setup.bRequest == usb_SET_IDLE {
+				sendZlp()
+				ok = true
 			}
 		}
 
@@ -2277,6 +2309,9 @@ func handleUSBIRQ(interrupt.Interrupt) {
 				if i == usb_CDC_ENDPOINT_IN {
 					USB.waitTxc = false
 				}
+			case usb_HID_ENDPOINT_IN:
+				setEPINTFLAG(i, sam.USB_DEVICE_ENDPOINT_EPINTFLAG_TRCPT1)
+				waitHidTxc = false
 			}
 		}
 	}
@@ -2374,7 +2409,7 @@ func handleStandardSetup(setup usbSetup) bool {
 			}
 		}
 
-		sendUSBPacket(0, buf)
+		sendUSBPacket(0, buf, setup.wLength)
 		return true
 
 	case usb_CLEAR_FEATURE:
@@ -2430,7 +2465,7 @@ func handleStandardSetup(setup usbSetup) bool {
 
 	case usb_GET_CONFIGURATION:
 		buff := []byte{usbConfiguration}
-		sendUSBPacket(0, buff)
+		sendUSBPacket(0, buff, setup.wLength)
 		return true
 
 	case usb_SET_CONFIGURATION:
@@ -2447,6 +2482,11 @@ func handleStandardSetup(setup usbSetup) bool {
 			// Enable interrupt for CDC data messages from host
 			setEPINTENSET(usb_CDC_ENDPOINT_OUT, sam.USB_DEVICE_ENDPOINT_EPINTENSET_TRCPT0)
 
+			// Enable interrupt for HID messages from host
+			if hidCallback != nil {
+				setEPINTENSET(usb_HID_ENDPOINT_IN, sam.USB_DEVICE_ENDPOINT_EPINTENSET_TRCPT1)
+			}
+
 			sendZlp()
 			return true
 		} else {
@@ -2455,7 +2495,7 @@ func handleStandardSetup(setup usbSetup) bool {
 
 	case usb_GET_INTERFACE:
 		buff := []byte{usbSetInterface}
-		sendUSBPacket(0, buff)
+		sendUSBPacket(0, buff, setup.wLength)
 		return true
 
 	case usb_SET_INTERFACE:
@@ -2481,7 +2521,7 @@ func cdcSetup(setup usbSetup) bool {
 			b[5] = byte(usbLineInfo.bParityType)
 			b[6] = byte(usbLineInfo.bDataBits)
 
-			sendUSBPacket(0, b[:])
+			sendUSBPacket(0, b[:], setup.wLength)
 			return true
 		}
 	}
@@ -2524,9 +2564,32 @@ func cdcSetup(setup usbSetup) bool {
 	return false
 }
 
+// SendUSBHIDPacket sends a packet for USBHID (interrupt / in).
+func SendUSBHIDPacket(ep uint32, data []byte) bool {
+	if waitHidTxc {
+		return false
+	}
+
+	sendUSBPacket(ep, data, 0)
+
+	// clear transfer complete flag
+	setEPINTFLAG(ep, sam.USB_DEVICE_ENDPOINT_EPINTFLAG_TRCPT1)
+
+	// send data by setting bank ready
+	setEPSTATUSSET(ep, sam.USB_DEVICE_ENDPOINT_EPSTATUSSET_BK1RDY)
+
+	waitHidTxc = true
+
+	return true
+}
+
 //go:noinline
-func sendUSBPacket(ep uint32, data []byte) {
-	copy(udd_ep_in_cache_buffer[ep][:], data)
+func sendUSBPacket(ep uint32, data []byte, maxsize uint16) {
+	l := uint16(len(data))
+	if 0 < maxsize && maxsize < l {
+		l = maxsize
+	}
+	copy(udd_ep_in_cache_buffer[ep][:], data[:l])
 
 	// Set endpoint address for sending data
 	usbEndpointDescriptors[ep].DeviceDescBank[1].ADDR.Set(uint32(uintptr(unsafe.Pointer(&udd_ep_in_cache_buffer[ep]))))
@@ -2536,7 +2599,7 @@ func sendUSBPacket(ep uint32, data []byte) {
 
 	// set byte count, which is total number of bytes to be sent
 	usbEndpointDescriptors[ep].DeviceDescBank[1].PCKSIZE.ClearBits(usb_DEVICE_PCKSIZE_BYTE_COUNT_Mask << usb_DEVICE_PCKSIZE_BYTE_COUNT_Pos)
-	usbEndpointDescriptors[ep].DeviceDescBank[1].PCKSIZE.SetBits(uint32((len(data) & usb_DEVICE_PCKSIZE_BYTE_COUNT_Mask) << usb_DEVICE_PCKSIZE_BYTE_COUNT_Pos))
+	usbEndpointDescriptors[ep].DeviceDescBank[1].PCKSIZE.SetBits((uint32(l) & usb_DEVICE_PCKSIZE_BYTE_COUNT_Mask) << usb_DEVICE_PCKSIZE_BYTE_COUNT_Pos)
 }
 
 func receiveUSBControlPacket() ([cdcLineInfoSize]byte, error) {
