@@ -26,19 +26,19 @@ func GOROOT() string {
 }
 
 // Copy size bytes from src to dst. The memory areas must not overlap.
-// Calls to this function are converted to LLVM intrinsic calls such as
-// llvm.memcpy.p0i8.p0i8.i32(dst, src, size, false).
+// This function is implemented by the compiler as a call to a LLVM intrinsic
+// like llvm.memcpy.p0i8.p0i8.i32(dst, src, size, false).
 func memcpy(dst, src unsafe.Pointer, size uintptr)
 
 // Copy size bytes from src to dst. The memory areas may overlap and will do the
 // correct thing.
-// Calls to this function are converted to LLVM intrinsic calls such as
-// llvm.memmove.p0i8.p0i8.i32(dst, src, size, false).
+// This function is implemented by the compiler as a call to a LLVM intrinsic
+// like llvm.memmove.p0i8.p0i8.i32(dst, src, size, false).
 func memmove(dst, src unsafe.Pointer, size uintptr)
 
 // Set the given number of bytes to zero.
-// Calls to this function are converted to LLVM intrinsic calls such as
-// llvm.memset.p0i8.i32(ptr, 0, size, false).
+// This function is implemented by the compiler as a call to a LLVM intrinsic
+// like llvm.memset.p0i8.i32(ptr, 0, size, false).
 func memzero(ptr unsafe.Pointer, size uintptr)
 
 // This intrinsic returns the current stack pointer.
@@ -74,3 +74,14 @@ func nanotime() int64 {
 func os_sigpipe() {
 	runtimePanic("too many writes on closed pipe")
 }
+
+// LockOSThread wires the calling goroutine to its current operating system thread.
+// Stub for now
+// Called by go1.18 standard library on windows, see https://github.com/golang/go/issues/49320
+func LockOSThread() {
+}
+
+// UnlockOSThread undoes an earlier call to LockOSThread.
+// Stub for now
+func UnlockOSThread() {
+}
