@@ -1,3 +1,4 @@
+//go:build fe310
 // +build fe310
 
 // This file implements target-specific things for the FE310 chip as used in the
@@ -98,6 +99,18 @@ func putchar(c byte) {
 	machine.Serial.WriteByte(c)
 }
 
+func getchar() byte {
+	for machine.Serial.Buffered() == 0 {
+		Gosched()
+	}
+	v, _ := machine.Serial.ReadByte()
+	return v
+}
+
+func buffered() int {
+	return machine.Serial.Buffered()
+}
+
 var timerWakeup volatile.Register8
 
 func ticks() timeUnit {
